#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from jpype import JPackage, java
import sys
from . import common

class MroTestCase(common.JPypeTestCase):
    def testMro(self):
        C = JPackage('jpype.mro').C

    def testMultipleInterfaces(self):
        j = JPackage("jpype").mro.MultipleInterfaces
        myinstance = j()



from jpype import *
import time

def run():
    print 'Thread started'
    try:
        print repr(java.awt.Frame)
        javax.swing.JFrame("Test Frame").setVisible(True)
        shutdownGuiEnvironment()
    except JException, ex :
        print ex


startJVM(getDefaultJVMPath())

setupGuiEnvironment(run)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import jpype
from jpype import java, JObject, JPackage, JString
from . import common
#import os
#import sys

class ObjectWrapperTestCase(common.JPypeTestCase):
    def testCallOverloads(self):
        # build the harness
        h = JPackage("jpype.objectwrapper").Test1()

        o = java.lang.Integer(1)
        self.assertEqual(h.Method1(JObject(o, java.lang.Number)), 1)
        self.assertEqual(h.Method1(o), 2)
        self.assertEqual(h.Method1(JObject(java.lang.Integer(1),
                                           java.lang.Object)), 3)
        self.assertEqual(h.Method1(JString("")), 4)

    def testDefaultTypeNameString(self):
        self.assertEqual(JObject("123").typeName, "java.lang.String")

    def testDefaultTypeNameBoolean(self):
        self.assertEqual(JObject(True).typeName, "java.lang.Boolean")
        self.assertEqual(JObject(False).typeName, "java.lang.Boolean")

    def testPassingClassTypeSucceeds(self):
        h = JPackage("jpype.objectwrapper").Test1()
        # Select a convenient java.lang.Class object
        class_obj = h.getClass()

        # Check that funneling Class obj through java doesn't convert to null
        result = h.ReturnObject(class_obj)

        self.assertEqual(class_obj, result)
        self.assertNotEqual(result, None)

    @unittest.skip("This seems to be a bug in _jwrapper.py _getDefaultTypeName")
    def testDefaultTypeNameJavaClass(self):
        o = java.lang.String
        self.assertEqual(JObject(o).typeName, "java.lang.Class")

#     def testMakeSureWeCanLoadAllClasses(self):
#         def get_system_jars():
#             for dirpath,_,files in os.walk(jpype.java.lang.System.getProperty("java.home")):
#                 for file in files:
#                     if file.endswith('.jar'):
#                         yield (os.path.join(dirpath,file))
#         for jar in get_system_jars():
#             classes = [x.getName() for x in jpype.java.util.jar.JarFile(jar).entries() if x.getName().endswith('.class')]
#             classes = [x.replace('/','.')[:-6] for x in classes]
#             for clazz in classes:
#                 try:
#                     jpype.JClass(clazz)
#                 except jpype.JavaException as exception:
#                     if not 'not found' in exception.message():
#                         print(clazz)
#                         print (exception.message())
#                         #print (exception.stacktrace())
#                 except:
#                     print(sys.exc_info()[0])
#                     pass
        


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from jpype import JPackage, java, JFloat, JByte, JShort, JInt, JLong
from . import common
import sys

class NumericTestCase(common.JPypeTestCase):
    def testMathAbs(self):
        self.assertEqual(java.lang.Math.abs(-10), 10)

    def testDoubleConversion(self):
        f = java.lang.Float.MAX_VALUE * 2
        jpype = JPackage("jpype")
        self.assertTrue(jpype.numeric.NumericTest.doubleIsTwiceMaxFloat(f))

    def testDoubleIsProperlyConverted(self):
        self.assertTrue(java.lang.Double.POSITIVE_INFINITY != 0.0)
        self.assertTrue(java.lang.Double.MAX_VALUE != 0.0)
        self.assertTrue(java.lang.Double.NaN != 0.0)
        self.assertTrue(java.lang.Double.NEGATIVE_INFINITY != 0.0)

    def testNegativeJFloatWrapper(self):
        f = JFloat(-1)

    def checkJWrapper(self, min_value, max_value, javawrapper, jwrapper, expected=TypeError):
        self.assertEqual(max_value, javawrapper(max_value).longValue())
        f = jwrapper(max_value)
        self.assertEqual(max_value, javawrapper(f).longValue())

        self.assertEqual(min_value, javawrapper(min_value).longValue())
        f = jwrapper(min_value)
        self.assertEqual(min_value, javawrapper(f).longValue())
        
        self.assertRaises(expected, javawrapper, max_value+1)
        self.assertRaises(expected, jwrapper, max_value+1)
        self.assertRaises(expected, javawrapper, min_value-1)
        self.assertRaises(expected, jwrapper, min_value-1)

        # test against int overflow
        if(max_value < 2**32):
            self.assertRaises(expected, javawrapper, 2**32)
            self.assertRaises(expected, jwrapper, 2**32)

    def testJCharWrapper(self):
        self.checkJWrapper(-2**7, 2**7-1, java.lang.Byte, JByte)

    def testJByteWrapper(self):
        self.checkJWrapper(-2**7, 2**7-1, java.lang.Byte, JByte)

    def testJShortWrapper(self):
        self.checkJWrapper(-2**15, 2**15-1, java.lang.Short, JShort)

    def testJIntWrapper(self):
        self.checkJWrapper(-2**31, 2**31-1, java.lang.Integer, JInt)

    def testJLongWrapper(self):
        self.checkJWrapper(-2**63, 2**63-1, java.lang.Long, JLong, OverflowError)
        
    def testJFloatWrapper(self):
        jwrapper = JFloat
        javawrapper = java.lang.Float
        jwrapper(float(2**127))
        javawrapper(float(2**127))
        self.assertRaises(TypeError, jwrapper, float(2**128))
        self.assertRaisesRegexp(RuntimeError, 'No matching overloads found', javawrapper, 5) # no conversion from int?

        # this difference might be undesirable, 
        # a double bigger than maxfloat passed to java.lang.Float turns into infinity 
        self.assertEquals(float('inf'), javawrapper(float(2**128)).doubleValue())
        self.assertEquals(float('-inf'), javawrapper(float(-2**128)).doubleValue())
                



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
import _jpype
from . import _jclass

_PROPERTY_ACCESSOR_PREFIX_LEN = 3

def _initialize() :
    _jclass.registerClassCustomizer(PropertiesCustomizer())

def _extract_accessor_pairs(members):
    """Extract pairs of corresponding property access methods
    (getter and setter) from a Java class's members (attributes).

    If a public method with a property's name exists no pair for
    that property will be extracted.

    Returns a dictionary with the property name as key and a tuple
    of (getter method, setter method) as value. A tuple element
    value might be `None` if only a getter or only a setter
    exists.
    """
    accessor_pairs = {}

    for name, member in members.items():
        if not (len(name) > _PROPERTY_ACCESSOR_PREFIX_LEN \
                        and _is_java_method(member)):
            continue
        access, rest = ( name[:_PROPERTY_ACCESSOR_PREFIX_LEN],
                         name[_PROPERTY_ACCESSOR_PREFIX_LEN:] )
        property_name = rest[:1].lower() + rest[1:]
        if property_name in members:
            if _is_java_method(members[property_name]):
                continue
        if access == 'get' and member.isBeanAccessor():
            try:
                pair = accessor_pairs[property_name]
                pair[0] = member
            except KeyError:
                accessor_pairs[property_name] = [member, None]
        elif access == 'set' and member.isBeanMutator():
            try:
                pair = accessor_pairs[property_name]
                pair[1] = member
            except KeyError:
                accessor_pairs[property_name] = [None, member]
    return accessor_pairs

def _is_java_method(attribute):
    return isinstance(attribute, _jpype._JavaMethod)

class PropertiesCustomizer(object) :
    def canCustomize(self, name, jc) :
        return True

    def customize(self, class_name, jc, bases, members) :
        accessor_pairs = _extract_accessor_pairs(members)
        for attr_name, (getter, setter) in accessor_pairs.items():
            if attr_name in members:
                if not getter:
                    # add default getter if we
                    # only have a setter
                    getter = members[attr_name].fget
                elif not setter:
                    # add default setter if we
                    # only have a getter
                    setter = members[attr_name].fset
            members[attr_name] = property(getter, setter)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from jpype import JException, java, JavaException, JProxy, JPackage
import traceback
try:
    import unittest2 as unittest
except ImportError:
    import unittest
from . import common

def throwIOException():
    raise java.io.IOException.PYEXC("Test throw")

def throwByJavaException():
    JPackage('jpype').exc.ExceptionTest.throwIOException()

class ExceptionTestCase(common.JPypeTestCase):
    def testExceptionThrown(self):
        try:
            self.jpype.exc.ExceptionTest.throwRuntime()
            self.fail()
        except JavaException as ex:
            self.assertIs(ex.javaClass(), java.lang.RuntimeException)
            self.assertEqual('Foo', ex.message())
            trace = ex.stacktrace()
            self.assertTrue(trace.startswith('java.lang.RuntimeException: Foo'))

    def testExceptionByJavaClass(self):
        try:
            self.jpype.exc.ExceptionTest.throwRuntime()
            self.fail()
        except JException(java.lang.RuntimeException) as ex:
            self.assertIs(ex.javaClass(), java.lang.RuntimeException)
            self.assertEqual('Foo', ex.message())
            trace = ex.stacktrace()
            self.assertTrue(trace.startswith('java.lang.RuntimeException: Foo'))

    @unittest.skip("Throwing specific Java exception from Python doesn't work")
    def testThrowException(self):
        d = {"throwIOException": throwIOException, }
        p = JProxy(self.jpype.exc.ExceptionThrower, dict=d)
        self.assertTrue(self.jpype.exc.ExceptionTest.delegateThrow(p))

    def testThrowException3(self):
        d = {"throwIOException": throwByJavaException, }
        p = JProxy(self.jpype.exc.ExceptionThrower, dict=d)

        self.assertTrue(self.jpype.exc.ExceptionTest.delegateThrow(p))

    def testExceptionPYEXCName(self):
        e = self.jpype.exc.ChildTestException()
        name = "jpype.exc.ChildTestExceptionPyRaisable"
        self.assertEqual(name, e.PYEXC.__name__)

    def testExceptionInstanceof(self):
        e = self.jpype.exc.ChildTestException()
        self.assertIsInstance(e, self.jpype.exc.ParentTestException)

    def testExceptionPYEXCInstanceof(self):
        e = self.jpype.exc.ChildTestException()
        self.assertTrue(issubclass(e.PYEXC,
                                   self.jpype.exc.ParentTestException.PYEXC))

    def testThrowChildExceptionFromCatchJExceptionParentClass(self):
        try:
            self.jpype.exc.ExceptionTest.throwChildTestException()
            self.fail()
        except JException(self.jpype.exc.ParentTestException) as ex:
            self.assertIsInstance(ex, self.jpype.exc.ChildTestException.PYEXC)



# part of JPype1; author Martin K. Scherer; 2014
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import mock

from jpype._jvmfinder import *
from jpype._linux import *
from jpype._darwin import *

import sys

class JVMFinderTest(unittest.TestCase):
    """
    test some methods to obtain a jvm.
    TODO: add windows (need to mock registry)
    """

    def test_find_libjvm(self):
        """
        test JVMFinder.find_libjvm does not return broken jvm implementation.
        """
        walk_fake = [('jre', ('lib',), ()),
                     ('jre/lib', ('amd64',), ()),
                     ('jre/lib/amd64',
                      ('cacao', 'jamvm', 'server'), ()),
                     ('jre/lib/amd64/cacao',
                      ('',), ('libjvm.so',)),
                     ('jre/lib/amd64/jamvm',
                      ('',), ('libjvm.so',)),
                     ('jre/lib/amd64/server',
                      ('',), ('libjvm.so',)),
                     ]
        with mock.patch('os.walk') as mockwalk:
            # contains broken and working jvms
            mockwalk.return_value = walk_fake

            finder = LinuxJVMFinder()
            p = finder.find_libjvm('arbitrary java home')
            self.assertEqual(
                p, os.path.join('jre/lib/amd64/server','libjvm.so'), 'wrong jvm returned')

        with mock.patch('os.walk') as mockwalk:
            # contains only broken jvms, since server impl is removed
            walk_fake[-1] = ((), (), (),)
            mockwalk.return_value = walk_fake

            finder = LinuxJVMFinder()
            with self.assertRaises(JVMNotSupportedException) as context:
                finder.find_libjvm('arbitrary java home')

    @mock.patch('os.walk')
    @mock.patch('os.path.exists')
    @mock.patch('os.path.realpath')
    def test_get_from_bin(self, mock_real_path, mock_path_exists, mock_os_walk):
        """
        test _get_from_bin method (used in linux and darwin)
        '/usr/bin/java' => some jre/jdk path
        """
        java_path = '/usr/lib/jvm/java-6-openjdk-amd64/bin/java'

        mock_os_walk.return_value = [
            ('/usr/lib/jvm/java-6-openjdk-amd64/jre/lib/amd64/server', ('',), ('libjvm.so',))]
        mock_path_exists.return_value = True
        mock_real_path.return_value = '/usr/lib/jvm/java-6-openjdk-amd64/bin/java'

        finder = LinuxJVMFinder()
        p = finder._get_from_bin()

        self.assertEqual(
            p, os.path.join('/usr/lib/jvm/java-6-openjdk-amd64/jre/lib/amd64/server','libjvm.so'))

    @unittest.skipIf(sys.version_info[:2] == (2, 6), "skip on py26")
    @mock.patch('platform.mac_ver')
    def test_javahome_binary_py27(self, mock_mac_ver):
        # this version has java_home binary
        mock_mac_ver.return_value = ('10.6.8', '', '')

        expected = '/System/Library/Java/JavaVirtualMachines/1.6.0.jdk/Contents/Home\n'

        finder = DarwinJVMFinder()

        # fake check_output
        with mock.patch('subprocess.check_output') as mock_checkoutput:
            mock_checkoutput.return_value = expected
            p = finder._javahome_binary()

            self.assertEqual(p, expected.strip())

        # this version has no java_home binary
        mock_mac_ver.return_value = ('10.5', '', '')
        p = finder._javahome_binary()

        self.assertEqual(p, None)

    @unittest.skipUnless(sys.version_info[:2] == (2, 6), "only py26")
    @mock.patch('platform.mac_ver')
    def test_javahome_binary_py26(self, mock_mac_ver):
        # this version has java_home binary
        mock_mac_ver.return_value = ('10.6.8', '', '')
        expected = '/System/Library/Java/JavaVirtualMachines/1.6.0.jdk/Contents/Home\n'

        finder = DarwinJVMFinder()

        # fake check_output
        with mock.patch('subprocess.Popen') as mock_popen:
            class proc:
                def communicate(self):
                    return (expected, )
            mock_popen.return_value = proc()

            p = finder._javahome_binary()

            self.assertEqual(p.strip(), expected.strip())

        # this version has no java_home binary
        mock_mac_ver.return_value = ('10.5', '', '')
        p = finder._javahome_binary()

        self.assertEqual(p, None)

if __name__ == '__main__':
    unittest.main()



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from os import path
import time
from xml.dom import minidom

def output(el, prefix="") :
    if el.nodeType != el.ELEMENT_NODE :
        return

    #print prefix, "<", el.tagName,

    atts = el.attributes
    for i in range(atts.length) :
        a = atts.item(i);
        #print a.nodeName, '="%s"' % a.nodeValue,
    #print '>'

    nl = el.childNodes
    for i in range(nl.length) :
        output(nl.item(i), prefix+"  ")


    #print prefix, "</", el.tagName, ">"

t = time.time()
count = 30
for i in range(count) :
    doc = minidom.parse(path.join(path.dirname(__file__), "sample", "big.xml"))

    el = doc.documentElement
    output(el)

t2 = time.time()
print count, "iterations in", t2-t, "seconds"



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
import unittest

import jpype
import logging
import os.path
import pkgutil
import sys

def suite() :
    loader = unittest.defaultTestLoader
    if len(sys.argv) > 1:
        names = sys.argv[1:]
        test_suite = loader.loadTestsFromNames(names)
    else:
        import jpypetest
        pkgpath = os.path.dirname(jpypetest.__file__)
        names = ["jpypetest.%s" % name for _, name,
                 _ in pkgutil.iter_modules([pkgpath])]
        test_suite = loader.loadTestsFromNames(names)
    return test_suite

def runTest() :
    runner = unittest.TextTestRunner()
    result = runner.run(suite())

    if jpype.isJVMStarted():
        jpype.shutdownJVM()
    if not result.wasSuccessful():
        sys.exit(1)

if __name__ == '__main__' :
    logging.basicConfig(level=logging.INFO)
    runTest()



from . import common

class PropertiesTestCase(common.JPypeTestCase):

    def setUp(self):
        super(PropertiesTestCase, self).setUp()
        self._bean = self.jpype.properties.TestBean()

    def testPropertyPublicMethodOverlap(self):
        self._bean.setProperty1("val")
        self.assertEqual("getsetval", self._bean.getProperty1())

    def testPublicMethodPropertyOverlap(self):
        self.assertEqual("method", self._bean.property1())

    def testPropertyProtectedMethodOverlapInvisibleAttribute(self):
        self._bean.property2 = "val"
        self.assertEqual("getsetval", self._bean.property2)

    def testProtectedMethodPropertyOverlapInvisibleAttribute(self):
        self.assertFalse(hasattr(self._bean.property2, '__call__'))

    def testPropertyProtectedMethodOverlapAttribute(self):
        self._bean.property3 = "val"
        self.assertEqual("getsetval", self._bean.property3)

    def testProtectedMethodPropertyOverlapAttribute(self):
        self.assertFalse(hasattr(self._bean.property3, '__call__'))

    def testPropertyProtectedMethodOverlapAttributeSet(self):
        self._bean.setProperty3("val")
        self.assertEqual("getsetval", self._bean.property3)

    def testPropertyProtectedMethodOverlapAttributeGet(self):
        self._bean.property3 = "val"
        self.assertEqual("getsetval", self._bean.getProperty3())

    def testPrivateAttributeNoThreeCharacterMethodMatchCollision(self):
        self._bean.property4 ="val"
        self.assertEqual("abcval", self._bean.abcProperty4())

    def testPropertyOnlySetter(self):
        self._bean.property5 = "val"
        self.assertEqual("returnsetval", self._bean.returnProperty5())

    def testPropertyOnlySetterSet(self):
        self._bean.setProperty5("val")
        self.assertEqual("setval", self._bean.property5)

    def testPropertyDifferentAttribute(self):
        self._bean.property6 = "val"
        self.assertEqual("getsetval", self._bean.property6)
        self.assertEqual("setval", self._bean.property7)

    def testProertyDifferentAttributeSet(self):
        self._bean.setProperty6("val")
        self.assertEqual("getsetval", self._bean.property6)
        self.assertEqual("setval", self._bean.property7)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
import jpype
from jpype import JString, java, JArray, JClass, JByte, JShort, JInt, JLong, JFloat, JDouble, JChar, JBoolean
import sys
import time
from . import common
from jpype._jwrapper import JObject

java = jpype.java

class OverloadTestCase(common.JPypeTestCase):
    def setUp(self):
        common.JPypeTestCase.setUp(self)
        self.__jp = self.jpype.overloads
        self._aclass = JClass('jpype.overloads.Test1$A')
        self._bclass = JClass('jpype.overloads.Test1$B')
        self._cclass = JClass('jpype.overloads.Test1$C')
        self._a = self._aclass()
        self._b = self._bclass()
        self._c = self._cclass()
        self._i1impl = JClass('jpype.overloads.Test1$I1Impl')()
        self._i2impl = JClass('jpype.overloads.Test1$I2Impl')()
        self._i3impl = JClass('jpype.overloads.Test1$I3Impl')()
        self._i4impl = JClass('jpype.overloads.Test1$I4Impl')()
        self._i5impl = JClass('jpype.overloads.Test1$I5Impl')()
        self._i6impl = JClass('jpype.overloads.Test1$I6Impl')()
        self._i7impl = JClass('jpype.overloads.Test1$I7Impl')()
        self._i8impl = JClass('jpype.overloads.Test1$I8Impl')()
        
    def testMostSpecificInstanceMethod(self):
        test1 = self.__jp.Test1()
        self.assertEquals('A', test1.testMostSpecific(self._a))
        self.assertEquals('B', test1.testMostSpecific(self._b))
        self.assertEquals('B', test1.testMostSpecific(self._c))
        self.assertEquals('B', test1.testMostSpecific(None))

    def testForceOverloadResolution(self):
        test1 = self.__jp.Test1()
        self.assertEquals('A', test1.testMostSpecific(JObject(self._c, self._aclass)))
        self.assertEquals('B', test1.testMostSpecific(JObject(self._c, self._bclass)))
        # JObject wrapper forces exact matches
        self.assertRaisesRegexp(RuntimeError, 'No matching overloads found', test1.testMostSpecific, JObject(self._c, self._cclass))
        self.assertEquals('A', test1.testMostSpecific(JObject(self._c, 'jpype.overloads.Test1$A')))
        self.assertEquals('B', test1.testMostSpecific(JObject(self._c, 'jpype.overloads.Test1$B')))
        # JObject wrapper forces exact matches
        self.assertRaisesRegexp(RuntimeError, 'No matching overloads found', test1.testMostSpecific, JObject(self._c, 'jpype.overloads.Test1$C'))
        
    def testVarArgsCall(self):
        test1 = self.__jp.Test1()
        self.assertEquals('A,B...', test1.testVarArgs(self._a,[]))
        self.assertEquals('A,B...', test1.testVarArgs(self._a, None))
        self.assertEquals('A,B...', test1.testVarArgs(self._a, [self._b]))
        self.assertEquals('A,A...', test1.testVarArgs(self._a, [self._a]))
        self.assertEquals('A,B...', test1.testVarArgs(self._a, [self._b,self._b]))
        self.assertEquals('B,B...', test1.testVarArgs(self._b, [self._b,self._b]))
        self.assertEquals('B,B...', test1.testVarArgs(self._c, [self._c,self._b]))
        self.assertEquals('B,B...', test1.testVarArgs(self._c, JArray(self._cclass,1)([self._c,self._c])))
        self.assertEquals('B,B...', test1.testVarArgs(self._c, None))
        self.assertEquals('B,B...', test1.testVarArgs(None, None))
        
    def testPrimitive(self):
        test1 = self.__jp.Test1()
        intexpectation = 'int' if not sys.version_info[0] > 2 and sys.maxint == 2**31 - 1 else 'long'
        self.assertEquals(intexpectation, test1.testPrimitive(5))
        self.assertEquals('long', test1.testPrimitive(2**31))
        self.assertEquals('byte', test1.testPrimitive(JByte(5)))
        self.assertEquals('Byte', test1.testPrimitive(java.lang.Byte(5)))
        self.assertEquals('short', test1.testPrimitive(JShort(5)))
        self.assertEquals('Short', test1.testPrimitive(java.lang.Short(5)))
        self.assertEquals('int', test1.testPrimitive(JInt(5)))
        self.assertEquals('Integer', test1.testPrimitive(java.lang.Integer(5)))
        self.assertEquals('long', test1.testPrimitive(JLong(5)))
        self.assertEquals('Long', test1.testPrimitive(java.lang.Long(5)))
        self.assertEquals('float', test1.testPrimitive(JFloat(5)))
        self.assertEquals('Float', test1.testPrimitive(java.lang.Float(5.0)))
        self.assertEquals('double', test1.testPrimitive(JDouble(5)))
        self.assertEquals('Double', test1.testPrimitive(java.lang.Double(5.0)))
        self.assertEquals('boolean', test1.testPrimitive(JBoolean(5)))
        self.assertEquals('Boolean', test1.testPrimitive(java.lang.Boolean(5)))
        self.assertEquals('char', test1.testPrimitive(JChar('5')))
        self.assertEquals('Character', test1.testPrimitive(java.lang.Character('5')))

    def testInstanceVsClassMethod(self):
        # this behaviour is different than the one in java, so maybe we should change it?
        test1 = self.__jp.Test1()
        self.assertEquals('static B', self.__jp.Test1.testInstanceVsClass(self._c))
        self.assertEquals('instance A', test1.testInstanceVsClass(self._c))
        # here what would the above resolve to in java
        self.assertEquals('static B', self.__jp.Test1.testJavaInstanceVsClass())

    def testInterfaces1(self):
        test1 = self.__jp.Test1()
        self.assertRaisesRegexp(RuntimeError, 'Ambiguous overloads found', test1.testInterfaces1, self._i4impl)
        self.assertEquals('I2', test1.testInterfaces1(JObject(self._i4impl, 'jpype.overloads.Test1$I2')))
        self.assertEquals('I3', test1.testInterfaces1(JObject(self._i4impl, 'jpype.overloads.Test1$I3')))

    def testInterfaces2(self):
        test1 = self.__jp.Test1()
        self.assertEquals('I4', test1.testInterfaces2(self._i4impl))
        self.assertEquals('I2', test1.testInterfaces2(JObject(self._i4impl, 'jpype.overloads.Test1$I2')))
        self.assertEquals('I3', test1.testInterfaces2(JObject(self._i4impl, 'jpype.overloads.Test1$I3')))

    def testInterfaces3(self):
        test1 = self.__jp.Test1()
        self.assertRaisesRegexp(RuntimeError, 'Ambiguous overloads found', test1.testInterfaces3, self._i8impl)
        self.assertEquals('I4', test1.testInterfaces3(self._i6impl))
        self.assertRaisesRegexp(RuntimeError, 'No matching overloads found', test1.testInterfaces3, self._i3impl)

    def testInterfaces4(self):
        test1 = self.__jp.Test1()
        self.assertEquals('I8', test1.testInterfaces4(None))
        self.assertEquals('I1', test1.testInterfaces4(self._i1impl))
        self.assertEquals('I2', test1.testInterfaces4(self._i2impl))
        self.assertEquals('I3', test1.testInterfaces4(self._i3impl))
        self.assertEquals('I4', test1.testInterfaces4(self._i4impl))
        self.assertEquals('I5', test1.testInterfaces4(self._i5impl))
        self.assertEquals('I6', test1.testInterfaces4(self._i6impl))
        self.assertEquals('I7', test1.testInterfaces4(self._i7impl))
        self.assertEquals('I8', test1.testInterfaces4(self._i8impl))
    
    def testClassVsObject(self):
        test1 = self.__jp.Test1()
        self.assertEquals('Object', test1.testClassVsObject(self._i4impl))
        self.assertEquals('Object', test1.testClassVsObject(1))
        self.assertEquals('Class', test1.testClassVsObject(None))
        self.assertEquals('Class', test1.testClassVsObject(JClass('jpype.overloads.Test1$I4Impl')))
        self.assertEquals('Class', test1.testClassVsObject(JClass('jpype.overloads.Test1$I3')))
        
    def testStringArray(self):
        test1 = self.__jp.Test1()
        self.assertEquals('Object', test1.testStringArray(self._i4impl))
        self.assertEquals('Object', test1.testStringArray(1))
        self.assertRaisesRegexp(RuntimeError, 'Ambiguous overloads found', test1.testStringArray, None)
        self.assertEquals('String', test1.testStringArray('somestring'))
        self.assertEquals('String[]', test1.testStringArray([]))
        self.assertEquals('String[]', test1.testStringArray(['a','b']))
        self.assertEquals('String[]', test1.testStringArray(JArray(java.lang.String,1)(['a','b'])))
        self.assertEquals('Object', test1.testStringArray(JArray(JInt, 1)([1,2])))
        
    def testListVSArray(self):
        test1 = self.__jp.Test1()
        self.assertEquals('String[]', test1.testListVSArray(['a','b']))
        self.assertEquals('List<String>', test1.testListVSArray(jpype.java.util.Arrays.asList(['a','b'])))
        
    def testDefaultMethods(self):
        try:
            testdefault = JClass('jpype.overloads.Test1$DefaultC')()
        except:
            pass
        else:
            self.assertEquals('B', testdefault.defaultMethod())



#*****************************************************************************
#   Copyright 2013 Thomas Calmant
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************

import os

from . import _jvmfinder

# ------------------------------------------------------------------------------

class LinuxJVMFinder(_jvmfinder.JVMFinder):
    """
    Linux JVM library finder class
    """
    def __init__(self):
        """
        Sets up members
        """
        # Call the parent constructor
        _jvmfinder.JVMFinder.__init__(self)

        # Java bin file
        self._java = "/usr/bin/java"

        # Library file name
        self._libfile = "libjvm.so"

        # Predefined locations
        self._locations = ("/usr/lib/jvm", "/usr/java", "/opt/sun")

        # Search methods
        self._methods = (self._get_from_java_home,
                         self._get_from_bin,
                         self._get_from_known_locations)

    def _get_from_bin(self):
        """
        Retrieves the Java library path according to the real installation of
        the java executable

        :return: The path to the JVM library, or None
        """
        # Find the real interpreter installation path
        java_bin = os.path.realpath(self._java)
        if os.path.exists(java_bin):
            # Get to the home directory
            java_home = os.path.abspath(os.path.join(os.path.dirname(java_bin),
                                                     '..'))

            # Look for the JVM library
            return self.find_libjvm(java_home)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from os import path
import time
from jpype import *

startJVM(getDefaultJVMPath(), "-ea")

# XML test
Element = JPackage("org").w3c.dom.Element

def output(el, prefix="") :
    if not isinstance(el, Element) :
        return

    #print prefix, "<", el.getTagName(),

    atts = el.getAttributes()
    for i in range(atts.getLength()) :
        a = atts.item(i);
        #print a.getNodeName(), '="%s"' % a.getNodeValue(),
    #print '>'

    nl = el.getChildNodes()
    for i in range(nl.getLength()) :
        output(nl.item(i), prefix+"  ")


    #print prefix, "</", el.getTagName(), ">"

t = time.time()
count = 30
for i in range(count) :
    build = javax.xml.parsers.DocumentBuilderFactory.newInstance().newDocumentBuilder()
    doc = build.parse(path.join(path.dirname(__file__), "sample", "big.xml"))

    el = doc.getDocumentElement()
    output(el)

t2 = time.time()
print count, "iterations in", t2-t, "seconds"

shutdownJVM()



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************

import sys

import _jpype
from . import _jclass
from ._jpackage import JPackage

if sys.version > '3':
    unicode = str
    long = int

def _initialize():
    _jpype.setWrapperClass(_JWrapper)
    _jpype.setStringWrapperClass(JString)

class _JWrapper(object):
    def __init__(self, v):
        if v is not None:
            self._value = _jpype.convertToJValue(self.typeName, v)
        else:
            self._value = None


class JByte(_JWrapper):
    typeName = "byte"

class JShort(_JWrapper):
    typeName = "short"

class JInt(_JWrapper):
    typeName = "int"

class JLong(_JWrapper):
    typeName = "long"

class JFloat(_JWrapper):
    typeName = "float"

class JDouble(_JWrapper):
    typeName = "double"

class JChar(_JWrapper):
    typeName = "char"

class JBoolean(_JWrapper):
    typeName = "boolean"

class JString(_JWrapper):
    typeName = "java.lang.String"

def _getDefaultTypeName(obj):
    if obj is True or obj is False:
        return 'java.lang.Boolean'

    if isinstance(obj, str) or isinstance(obj, unicode):
        return "java.lang.String"

    if isinstance(obj, int):
        return "java.lang.Integer"

    if isinstance(obj, long):
        return "java.lang.Long"

    if isinstance(obj, float):
        return "java.lang.Double"

    if isinstance(obj, _jclass._JavaClass):
        return obj.__javaclassname__

    if isinstance(obj, JPackage("java").lang.Class):
        return obj.__class__.__javaclass__.getName()

    if isinstance(obj, _JWrapper):
        return obj.typeName

    raise TypeError(
        "Unable to determine the default type of {0}".format(obj.__class__))

class JObject(_JWrapper):
    def __init__(self, v, tp=None):
        if tp is None:
            tp = _getDefaultTypeName(v)
        if isinstance(tp, _jclass._JavaClass):
            tp = tp.__javaclass__.getName()

        self.typeName = tp
        self._value = _jpype.convertToJValue(tp, v)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
import collections

from . import _jclass

class _WrappedIterator(object):
    """
    Wraps a Java iterator to respect the Python 3 iterator API
    """
    def __init__(self, iterator):
        self.iterator = iterator

    def __iter__(self):
        return self.iterator

    def __next__(self):
        return next(self.iterator)

    # Compatibility name
    next = __next__

def _initialize():
    _jclass.registerClassCustomizer(CollectionCustomizer())
    _jclass.registerClassCustomizer(ListCustomizer())
    _jclass.registerClassCustomizer(MapCustomizer())
    _jclass.registerClassCustomizer(IteratorCustomizer())
    _jclass.registerClassCustomizer(EnumerationCustomizer())

def isPythonSequence(v):
    if isinstance(v, collections.Sequence):
        if not hasattr(v.__class__, '__metaclass__') \
           or v.__class__.__metaclass__ is _jclass._JavaClass:
            return True
    return False

def _colLength(self):
    return self.size()

def _colIter(self):
    return _WrappedIterator(self.iterator())

def _colDelItem(self, i):
    return self.remove(i)

def _colAddAll(self, v):
    if isPythonSequence(v):
        r = False
        for i in v:
            r = self.add(i) or r
        return r
    else:
        return self._addAll(v)

def _colRemoveAll(self, v):
    if isPythonSequence(v):
        r = False
        for i in v:
            r = self.remove(i) or r
        return r
    else:
        return self._removeAll(v)

def _colRetainAll(self, v):
    if isPythonSequence(v):
        r = _jclass.JClass("java.util.ArrayList")(len(v))
        for i in v:
            r.add(i)
    else:
        r = v

    return self._retainAll(r)

class CollectionCustomizer(object):
    _METHODS = {
        '__len__': _colLength,
        '__iter__': _colIter,
        '__delitem__': _colDelItem,
    }

    def canCustomize(self, name, jc):
        if name == 'java.util.Collection':
            return True
        return jc.isSubclass('java.util.Collection')

    def customize(self, name, jc, bases, members):
        if name == 'java.util.Collection':
            members.update(CollectionCustomizer._METHODS)
        else:
            # AddAll is handled by List
            if (not jc.isSubclass("java.util.List")) and 'addAll' in members:
                members['_addAll'] = members['addAll']
                members['addAll'] = _colAddAll
            if 'removeAll' in members:
                members['_removeAll'] = members['removeAll']
                members['removeAll'] = _colRemoveAll
            if 'retainAll' in members:
                members['_retainAll'] = members['retainAll']
                members['retainAll'] = _colRetainAll

def _listGetItem(self, ndx):
    if isinstance(ndx, slice):
        start = ndx.start
        stop = ndx.stop
        if start < 0:
            start = self.size() + start
        if stop < 0:
            stop = self.size() + stop
        return self.subList(start, stop)
    else:
        if ndx < 0:
            ndx = self.size() + ndx
        return self.get(ndx)

def _listSetItem(self, ndx, v):
    if isinstance(ndx, slice):
        start = ndx.start
        stop = ndx.stop
        if start < 0:
            start = self.size() + start
        if stop < 0:
            stop = self.size() + stop
        for i in range(start, stop):
            self.remove(start)
        if isinstance(v, collections.Sequence):
            ndx = start
            for i in v:
                self.add(ndx, i)
                ndx += 1
    else:
        if ndx < 0:
            ndx = self.size() + ndx
        self.set(ndx, v)

def _listAddAll(self, v, v2=None):
    if isPythonSequence(v):
        r = False
        if v2 is not None: # assume form (int, values)
            for i in range(len(v2)):
                r = r or self.add(v + i, v2[i])
        else:
            for i in v:
                r = self.add(i) or r
        return r
    else:
        return self._addAll(v)

class ListCustomizer(object):
    _METHODS = {
        '__setitem__': _listSetItem,
        '__getitem__': _listGetItem,
    }

    def canCustomize(self, name, jc):
        if name == 'java.util.List':
            return True
        return jc.isSubclass('java.util.List')

    def customize(self, name, jc, bases, members):
        if name == 'java.util.List':
            members.update(ListCustomizer._METHODS)
        else:
            if 'addAll' in members:
                members['_addAll'] = members['addAll']
                members['addAll'] = _listAddAll

def isPythonMapping(v):
    if isinstance(v, collections.Mapping):
        if not hasattr(v.__class__, '__metaclass__') or \
           v.__class__.__metaclass__ is _jclass._JavaClass:
            return True
    return False

def _mapLength(self):
    return self.size()

def _mapIter(self):
    return _WrappedIterator(self.keySet().iterator())

def _mapDelItem(self, i):
    return self.remove(i)

def _mapGetItem(self, ndx):
    return self.get(ndx)

def _mapSetItem(self, ndx, v):
    self.put(ndx, v)

def _mapPutAll(self, v):
    if isPythonMapping(v):
        for i in v:
            self.put(i, v[i])
    else:
        # do the regular method ...
        self._putAll(v)

class MapCustomizer(object):
    _METHODS = {
        '__len__': _mapLength,
        '__iter__': _mapIter,
        '__delitem__': _mapDelItem,
        '__getitem__': _mapGetItem,
        '__setitem__': _mapSetItem,
    }

    def canCustomize(self, name, jc):
        if name == 'java.util.Map':
            return True
        return jc.isSubclass('java.util.Map')

    def customize(self, name, jc, bases, members):
        if name == 'java.util.Map':
            members.update(MapCustomizer._METHODS)
        else:
            if "putAll" in members:
                members["_putAll"] = members["putAll"]
                members["putAll"] = _mapPutAll

def _iterCustomNext(self):
    if self.hasNext():
        return self._next()
    raise StopIteration


def _iterIteratorNext(self):
    if self.hasNext():
        return next(self)
    raise StopIteration

def _iterIter(self):
    return self

class IteratorCustomizer(object):
    _METHODS = {
        '__iter__': _iterIter,
        '__next__': _iterCustomNext,
    }

    def canCustomize(self, name, jc):
        if name == 'java.util.Iterator':
            return True
        return jc.isSubclass('java.util.Iterator')

    def customize(self, name, jc, bases, members):
        if name == 'java.util.Iterator':
            members.update(IteratorCustomizer._METHODS)
        elif jc.isSubclass('java.util.Iterator'):
            __next__ = 'next' if 'next' in members else '__next__'
            members['_next'] = members[__next__]
            members[__next__] = _iterCustomNext

def _enumNext(self):
    if self.hasMoreElements():
        return self.nextElement()
    raise StopIteration

def _enumIter(self):
    return self


class EnumerationCustomizer(object):
    _METHODS = {
        'next': _enumNext,
        '__next__': _enumNext,
        '__iter__': _enumIter,
    }

    def canCustomize(self, name, jc):
        return name == 'java.util.Enumeration'

    def customize(self, name, jc, bases, members):
        members.update(EnumerationCustomizer._METHODS)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from jpype import *
from . import common

try:
    import unittest2 as unittest
except ImportError:
    import unittest

def _testMethod1():
    return 33

def _testMethod2():
    return 32

def _testMethod3():
    return "Foo"

class C:
    def testMethod1(self):
        return 43

    def testMethod2(self):
        return 42

    def testMethod3(self):
        return "Bar"

    def write(self, bytes, start, length):
        return bytes, start, length

class ThreadCallbackImpl:
    def __init__(self):
        self.values = []

    def notifyValue(self, val):
        self.values.append(val)

class ProxyTestCase(common.JPypeTestCase):

    def setUp(self):
        super(ProxyTestCase, self).setUp()
        self.package = JPackage("jpype.proxy")
        self._triggers = self.package.ProxyTriggers

    def testProxyWithDict(self):
        d = {
            'testMethod1': _testMethod1,
            'testMethod2': _testMethod2,
        }
        itf1 = self.package.TestInterface1
        itf2 = self.package.TestInterface2
        proxy = JProxy([itf1, itf2], dict=d)

        result = self._triggers.testProxy(proxy)

        expected = ['Test Method1 = 33', 'Test Method2 = 32']
        self.assertSequenceEqual(result, expected)

    def testProxyWithDictInherited(self):
        d = {
            'testMethod2': _testMethod2,
            'testMethod3': _testMethod3,
        }
        itf3 = self.package.TestInterface3
        proxy = JProxy(itf3, dict=d)

        result = self._triggers.testProxy(proxy)

        expected = ['Test Method2 = 32', 'Test Method3 = Foo']
        self.assertSequenceEqual(result, expected)

    def testProxyWithInst(self):
        itf3 = self.package.TestInterface3
        c = C()
        proxy = JProxy(itf3, inst=c)

        result = self._triggers.testProxy(proxy)

        expected = ['Test Method2 = 42', 'Test Method3 = Bar']
        self.assertSequenceEqual(result, expected)

    def testProxyWithThread(self):
        itf = self.package.TestThreadCallback
        tcb = ThreadCallbackImpl()
        proxy = JProxy(itf, inst=tcb)

        self._triggers().testProxyWithThread(proxy)
        self.assertEqual(tcb.values, ['Waiting for thread start',
                                      '1', '2', '3',
                                      'Thread finished'])

    def testProxyWithArguments(self):
        itf2 = self.package.TestInterface2
        c = C()
        proxy = JProxy(itf2, inst=c)

        result = self._triggers().testCallbackWithParameters(proxy)

        bytes, start, length = result
        self.assertSequenceEqual(bytes, [1, 2 ,3 ,4])
        self.assertEqual(start, 12)
        self.assertEqual(length, 13)

    def testProxyWithMultipleInterface(self):
        itf1 = self.package.TestInterface1
        itf2 = self.package.TestInterface2
        c = C()
        proxy = JProxy([itf1, itf2], inst=c)

        result = self._triggers.testProxy(proxy)

        expected = ['Test Method1 = 43', 'Test Method2 = 42']
        self.assertSequenceEqual(result, expected)

    def testProxyWithMultipleInterfaceInherited(self):
        itf2 = self.package.TestInterface2
        itf3 = self.package.TestInterface3
        c = C()
        proxy = JProxy([itf2,itf3], inst=c)

        result = self._triggers().testCallbackWithParameters(proxy)

        bytes, start, length = result
        self.assertSequenceEqual(bytes, [1, 2 ,3 ,4])
        self.assertEqual(start, 12)
        self.assertEqual(length, 13)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************

from . import _jvmfinder
try:
    import _winreg as winreg
except ImportError:
    import winreg  # in Py3, winreg has been moved

# ------------------------------------------------------------------------------

class WindowsJVMFinder(_jvmfinder.JVMFinder):
    """
    Linux JVM library finder class
    """
    def __init__(self):
        """
        Sets up members
        """
        # Call the parent constructor
        _jvmfinder.JVMFinder.__init__(self)

        # Library file name
        self._libfile = "jvm.dll"

        # Search methods
        self._methods = (self._get_from_java_home,
                         self._get_from_registry)


    def _get_from_registry(self):
        """
        Retrieves the path to the default Java installation stored in the
        Windows registry

        :return: The path found in the registry, or None
        """
        try :
            jreKey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
                                r"SOFTWARE\JavaSoft\Java Runtime Environment")
            cv = winreg.QueryValueEx(jreKey, "CurrentVersion")
            versionKey = winreg.OpenKey(jreKey, cv[0])
            winreg.CloseKey(jreKey)

            cv = winreg.QueryValueEx(versionKey, "RuntimeLib")
            winreg.CloseKey(versionKey)

            return cv[0]

        except WindowsError:
            return None



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from jpype import JException, java, JavaException, JProxy, JClass
import os
import tempfile
import traceback
from . import common

class SerializationTestCase(common.JPypeTestCase):

    def setUp(self):
        super(SerializationTestCase, self).setUp()
        self.tempname = tempfile.mktemp()

    def tearDown(self):
        os.remove(self.tempname)

    def testSerialize(self):
        o = JClass("jpype.serial.SerializationTest")()
        fos = java.io.FileOutputStream(self.tempname)
        oos = java.io.ObjectOutputStream(fos)
        oos.writeObject(o)
        oos.flush()
        oos.close()
        fos.close()



# The following cannto work because JPype has no way to simulate the "caller's ClassLoader"
#    def testDeSerialize(self):
#        fis = java.io.FileInputStream(self.tempname)
#        ois = java.io.ObjectInputStream(fis)
#
#        o = ois.readObject()
#        ois.close()
#        fis.close()



#!/usr/bin/env python
"""
Checks for memory leak in the JVM when pushing array data from Python to Java.
"""

import pytest
import jpype

#!!! These settings are finely tuned !!!
#!!! DO NOT fiddle with them unless you know what you are doing !!!

#   Size of array to be copied.
ARRAY_SIZE = 4000000

#   Number of iterations to run.
#   - raise this value to increase efficacy of the memory leak test.
ITERATIONS = 10

#   Maximum size of JVM heap.
#   - sets a cap to allow memory leak detection.
MAX_JVM_HEAP_SIZE_BYTES = 128647168

def setup_module(module):
    #   Module-level setup.
    if not jpype.isJVMStarted():
        jvm_args = [
            '-Xmx%dM' % (MAX_JVM_HEAP_SIZE_BYTES // 1000 ** 2),
        ]
        jpype.startJVM(jpype.getDefaultJVMPath(), *jvm_args)
    module.JavaDoubleArray = jpype.JArray(jpype.JDouble, 1)

def test_memory_leak_fix():
    """
    This test raises java.lang.VirtualMachineErrorPyRaisable
    (java.lang.OutOfMemoryError: Java heap space) if the memory leak
    is present.

    """
    #   Check memory settings.
    rt = jpype.java.lang.Runtime.getRuntime()
    assert rt.maxMemory() == MAX_JVM_HEAP_SIZE_BYTES

    #   Perform leak test.
    for i in xrange(ITERATIONS):
        print 'iteration:', i
        py_list1 = [float(f) for f in xrange(ARRAY_SIZE)]
        j_array1 = JavaDoubleArray(py_list1)
        py_list2 = j_array1[:]
        assert py_list1 == py_list2

def test_jarray_basic_slicing_fix():
    jl1 = JavaDoubleArray([1., 2., 3.])
    assert list(jl1) == [1., 2., 3.]
    assert list(jl1[0:-1]) == [1., 2.]
    assert list(jl1[0:1]) == [1.]

def test_jarray_slice_copy_fix():
    jl1 = JavaDoubleArray([1., 2., 3.])
    pl1 = jl1[:]
    assert list(jl1) == pl1

def test_jarray_slice_assignment_fix():
    jl2 = JavaDoubleArray([1., 2., 3.])
    jl2[:] = [4., 5., 6.]
    assert list(jl2) == [4., 5., 6.]



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
KEYWORDS = set((
    "del", "for", "is", "raise",
    "assert", "elif", "from", "lambda", "return",
    "break", "else", "global", "not", "try",
    "class", "except", "if", "or", "while",
    "continue", "exec", "import", "pass", "yield",
    "def", "finally", "in", "print", "as", "None"
))



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from os import path
import time
from jpype import *

root = path.abspath(path.dirname(__file__))
startJVM(getDefaultJVMPath(), "-ea",
         "-Djava.class.path=%s" % path.join(root, "classes"))

# XML test
Element = JPackage("org").w3c.dom.Element

class ContentHandler(object) :
    def characters(self, ch, start, length) :
        pass

    def endDocument(self) :
        pass

    def endElement(self, namespaceURI, localName, qName) :
        pass

    def endPrefixMapping(self, prefix) :
        pass

    def ignorableWhitespace(self, ch, start, length) :
        pass

    def processingInstruction(self, target, data) :
        pass

    def setDocumentLocator(self, locator) :
        pass

    def skippedEntity(self, name) :
        pass

    def startDocument(self, ) :
        pass

    def startElement(self, namespaceURI, localName, qName, atts) :
        pass

    def startPrefixMapping(self, prefix, uri) :
        pass

t = time.time()
count = 30
for i in range(count) :
    DelegateHandler = JPackage("jpype.xml").DelegateHandler
    dh = DelegateHandler(None, None, JProxy("org.xml.sax.ContentHandler", inst=ContentHandler()), None)

    build = javax.xml.parsers.SAXParserFactory.newInstance().newSAXParser()
    build.parse(path.join(root, "sample", "big.xml"), dh)

t2 = time.time()
print count, "iterations in", t2-t, "seconds"

shutdownJVM()



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
import _jpype
from . import _jclass

_CLASSES = {}

def JException(t):
    return t.PYEXC

class JavaException(Exception):
    def __init__(self, *data):
        if isinstance(data[0], tuple) \
           and data[0][0] is _jclass._SPECIAL_CONSTRUCTOR_KEY:
            # this is wrapped:
            self.__javaobject__ = data[0][1]
            self.__javaclass__ = self.__javaobject__.__class__
        else:
            self.__javaclass__ = _jclass.JClass(self.__class__.JAVACLASSNAME)
            self.__javaobject__ = self.__javaclass__(*data)

        Exception.__init__(self, self.__javaobject__)

    def javaClass(self):
        return self.__javaclass__

    def message(self):
        return self.__javaobject__.getMessage()

    def stacktrace(self):
        StringWriter = _jclass.JClass("java.io.StringWriter")
        PrintWriter = _jclass.JClass("java.io.PrintWriter")
        sw = StringWriter()
        pw = PrintWriter(sw)

        self.__javaobject__.printStackTrace(pw)
        pw.flush()
        r = sw.toString()
        sw.close()
        return r

    def __str__(self):
        return self.__javaobject__.toString()

def _initialize():
    _jpype.setJavaExceptionClass(JavaException)

def _makePythonException(name, bc):
    if name in _CLASSES:
        return _CLASSES[name]

    if name == 'java.lang.Throwable':
        bases = (JavaException,)
    else:
        bases = (_makePythonException(bc.getName(), bc.getBaseClass()) ,)

    ec = type(name+"PyRaisable", bases, {'JAVACLASSNAME': name})

    _CLASSES[name] = ec
    return ec



from jpype import *
import time


startJVM(getDefaultJVMPath())
#startJVM("c:/tools/jdk1.4.2/jre/bin/server/jvm.dll")

arr = java.util.ArrayList()

# no matching overloads found for this line:
arr.addAll([str(x) for x in xrange(50)])

print arr

hmap = java.util.HashMap()

# no matching overloads found for this line:
hmap.putAll({5:6, 7:8, 'hello':'there'})

print hmap

#for x in xrange(5):
#    # this works:
#    hmap.put(str(x), str(x))
#    # but this doesn't:
#    hmap.put(str(x), x)
#
#
## this throws: AttributeError: 'java.util.HashMap' object has no attribute 'iterator'
#for x in hmap:
#    print x, hmap[x]



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************

import jpype
import array
import time
import os

def generateStringData(aSize):
    return ''.join(['a']*aSize)

DATA_SIZE = 5*1024*1024 # 5 MB



def runBaseline(data):
    print 'Running baseline test : converting a python string->array.array->JArray(JByte). size = ', len(data)/1024.0, 'kb'
    print '    Start time (no optimize) on my machine is 3.56 seconds.'
    start = time.time()

    #darr = array.array('b', DATA)
    arr_cls = jpype.JArray(jpype.JByte)
    java_arr = arr_cls(DATA)

    end = time.time()

    print '    test run in', (end-start), 'seconds.'

def runStringToByteBuffer(data):
    print 'Running String conversion to byte buffer. size = ', len(data)/1024.0, 'kb'
    start = time.time()

    bb = jpype.nio.convertToDirectBuffer(data)

    end = time.time()

    print '    test run in', (end-start), 'seconds.'

    jpype.JPackage("jpype").nio.NioReceive.receiveBuffer(bb)

def runStringToByteArray(data):
    print 'Running String conversion to byte array. size = ', len(data)/1024.0, 'kb'
    start = time.time()

    arr_cls = jpype.JArray(jpype.JByte)
    java_arr = arr_cls(data)

    end = time.time()

    print '    test run in', (end-start), 'seconds.'

root = os.path.abspath(os.path.dirname(__file__))
jpype.startJVM(jpype.getDefaultJVMPath(), "-ea", "-Xmx5M", "-verbose:gc", "-Djava.class.path=./classes%s%s%sclasses" % (os.pathsep, root, os.sep))

DELETED = False
class MyStr(str):
    def __del__(self):
        global DELETED
        print 'string got deleted'
        DELETED = True

def testStringMemory():
    print 'with keeping the data'
    data = MyStr('5' * 1024)
    print data
    buf = jpype.nio.convertToDirectBuffer(data)
#    print buf.get()
#    print buf.get()
#    print buf.get()

    print 'now deleting the data'
    del data
#    print buf.get()
#    print buf.get()
#    print buf.get()
#    print buf.get()

    print 'now deleting the buffer itself'
    del buf
    print 'now waiting for the string to get deleted'
    while not DELETED:
        time.sleep(1)

        print '.',
        jpype.JPackage("jpype").nio.NioReceive.allocSomeMemory()

testStringMemory()

#for i in range(1,5) :

#    DATA = generateStringData(DATA_SIZE*i)
#    runBaseline(DATA)
#    runStringToByteBuffer(DATA)
#    runStringToByteArray(DATA)

    # expressly delete data to test the GC ...
    #del DATA
    #for i in range(3) :
    #    print 'GC', i
    #    jpype.JClass("java.lang.System").gc();
    #    time.sleep(15)

jpype.shutdownJVM()



from jpype import *
import time

remote_pack="c:/tools/netbeean-remote-pack"

profiler_options = [
        "-agentpath:%s/lib/deployed/jdk15/windows/profilerinterface.dll=%s/lib,5140" % (remote_pack, remote_pack)
]

options = [
        '-verbose:gc',
        '-Xmx16m',
] #+ profiler_options

startJVM(getDefaultJVMPath(), *options)

class MyStr(str):
    def __del__(self):
        print 'string got deleted'

while True:
    buf = java.lang.String('5' * 1024 * 1024 * 5)
    buf = nio.convertToDirectBuffer(MyStr('5' * 1024 * 1024))
#       time.sleep(1)



from jpype import *
import time

remote_pack="c:/tools/netbeean-remote-pack"

profiler_options = [
    "-agentpath:%s/lib/deployed/jdk15/windows/profilerinterface.dll=%s/lib,5140" % (remote_pack, remote_pack)
]


options = [
    #'-verbose:gc',
    '-Xmx64m',
    '-Djava.class.path=classes'
] #+ profiler_options

cnt = 0

#setUsePythonThreadForDeamon(True)
startJVM(getDefaultJVMPath(), *options)

class MyStr(str):
    def __init__ (self, val):
        str.__init__(self, val)
        global cnt
        cnt += 1
        print 'created string', cnt

    def __del__(self):
        global cnt
        cnt -= 1
        print 'deleted string', cnt

receive = JClass("jpype.nio.NioReceive")

while True:
    # everything runs great with this line uncommented
    #p = JString('5' * 1024 * 1024)

    # with this line uncommented, the python strings aren't GC'd
    p = java.lang.StringBuffer(MyStr('5' * 1024 * 1024))

    # with this line uncommented, the JVM throws an OutOfMemoryError (not GC'ing the proxied java objects?),
    # but the python strings are being GC'd
    #p = java.lang.StringBuffer(JString(MyStr('5' * 1024 * 1024)))

    #
    # forget the direct buffer for now....
    #
    buf = nio.convertToDirectBuffer(MyStr('5' * 1024 * 1024 * 5))
    try :
        receive.receiveBufferWithException(buf)
    except :
        pass



from jpype import java, javax, JProxy

class AbstractAction(object):
    ACCELERATOR_KEY    = javax.swing.Action.ACCELERATOR_KEY
    ACTION_COMMAND_KEY = javax.swing.Action.ACTION_COMMAND_KEY
    DEFAULT            = javax.swing.Action.DEFAULT
    LONG_DESCRIPTION   = javax.swing.Action.LONG_DESCRIPTION
    MNEMONIC_KEY       = javax.swing.Action.MNEMONIC_KEY
    NAME               = javax.swing.Action.NAME
    SHORT_DESCRIPTION  = javax.swing.Action.SHORT_DESCRIPTION
    SMALL_ICON         = javax.swing.Action.SMALL_ICON

    def __init__(self, cb, name=None, icon=None):
        object.__init__(self)

        self.__proxy = JProxy(javax.swing.Action, inst=self)
        self.__values = {}
        self.__cb = cb
        self.__listeners = []
        self.__enabled = True

        if name is not None:
            self.putValue(AbstractAction.NAME, name)

        if icon is not None:
            self.putValue(AbstractAction.SMALL_ICON, icon)

    proxy = property(lambda self: self.__proxy)

    def addPropertyChangeListener(self, listener):
        self.__listeners.append(listener)

    def getValue(self, key):
        return self.__values.get(key, None)

    def isEnabled(self):
        if self.__enabled:
            return True
        return False

    def putValue(self, key, value):
        oldVal = self.__values.get(key, None)
        if oldVal != value:
            self.__values[key] = value
            self.__notify(key, oldVal, value)

    def removePropertyChangeListener(self, listener):
        self.__listeners.remove(listener)

    def setEnabled(self, b):
        if (b and not self.__enabled) or (not b and self.__enabled):
            self.__enabled = b
            self.__notify("enabled", java.lang.Boolean(not b),
                          java.lang.Boolean(b))

    def actionPerformed(self, ev):
        self.__cb(ev)

    def __notify(self, k, oldVal, newVal):
        ev = java.beans.PropertyChangeEvent(self.__proxy, k, oldVal, newVal)
        for i in self.__listeners:
            i.propertyChange(ev)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
import jpype
from jpype import JString, java, JArray
import sys
import time
from . import common

class AttributeTestCase(common.JPypeTestCase):
    def setUp(self):
        common.JPypeTestCase.setUp(self)
        self.__jp = self.jpype.attr

    def testWithBufferStrategy(self):
        j = jpype.JPackage("jpype").attr.ClassWithBuffer
        self.assertIsNone(j().bufferStrategy)

    def testCallOverloadedMethodWithCovariance(self):
        # This is a JDk5-specific problem.
        h = jpype.java.lang.StringBuffer()
        h.delete(0, 0)

    def testCallStaticString(self):
        h = self.__jp.Test1()
        v = h.testString(JString("abcd"), JString("abcde"))

        self.assertEqual(v[0], 'abcd')
        self.assertEqual(v[1], 'abcde')

    def testCallStaticUnicodeString(self):
        h = self.__jp.Test1()
        v = h.testString(JString(u"abcd"), JString(u"abcde"))

        self.assertEqual(v[0], 'abcd')
        self.assertEqual(v[1], 'abcde')

    def testCallString(self):
        v = self.__jp.Test1.testStaticString("a", "b")
        self.assertEqual(v[0], 'a')
        self.assertEqual(v[1], 'b')

    def testCallUnicodeString(self):
        v = self.__jp.Test1.testStaticString(u"a", u"b")
        self.assertEqual(v[0], 'a')
        self.assertEqual(v[1], 'b')

    def testCallStringWithNone(self):
        v = self.__jp.Test1.testStaticString("a", None)
        self.assertEqual(v[0], 'a')
        self.assertIsNone(v[1])

    def testWithHolder(self):
        holder = self.__jp.Holder()
        holder.f = "ffff"
        self.assertEqual(holder.f, 'ffff')
        result = self.__jp.Test1.testStaticHolder(holder)
        self.assertEqual(result, 'ffff')

    def testWithSubHolder(self):
        h2 = self.__jp.SubHolder()
        h2.f = "subholder"
        result = self.__jp.Test1.testStaticHolder(h2)
        self.assertEqual(result, 'subholder')

    def testCallWithArray(self):
        h2 = self.__jp.Test1()
        StringArray = JArray(JString)
        v = StringArray(["Foo", "bar"])
        t = self.__jp.Test1()
        result = t.testStringArray(v)
        self.assertSequenceEqual(["Foo", "bar"], result)

    def testGetStaticValue(self):
        self.assertEqual(str(self.__jp.Test1.objectValue), "234")

    def testGetStaticByInstance(self):
        h = self.__jp.Test1()
        self.assertEqual(str(h.objectValue), "234")

    def testGetNonStatic(self):
        h = self.__jp.Test1()
        self.assertEqual(h.stringValue, "Foo")

    def testSetStaticValue(self):
        self.__jp.Test1.objectValue = java.lang.Integer(43)
        self.assertEqual(str(self.__jp.Test1.objectValue), "43")
        self.__jp.Test1.reset()

    def testSetNonStaticValue(self):
        h = self.__jp.Test1()
        h.stringValue="bar"
        self.assertEqual(h.stringValue, "bar")

    def testReturnSubClass(self):
        h = self.__jp.Test1()
        v = h.getSubClass()
        self.assertIsInstance(v, self.__jp.SubHolder)

    def testCallWithClass(self):
        h = self.__jp.Test1()
        h.callWithClass(java.lang.Comparable)

    def testCallSuperclassMethod(self):
        h = self.__jp.Test2()
        h.test2Method()
        h.test1Method()

    def testCallWithLong(self):
        h = self.__jp.Test1()
        if sys.version > '3':
            l = int(123)
        else:
            l = long(123)

        h.setByte(l)
        self.assertEqual(l, h.mByteValue)
        h.setShort(l)
        self.assertEqual(l, h.mShortValue)
        h.setInt(l)
        self.assertEqual(l, h.mIntValue)
        h.setLong(l)
        self.assertEqual(l, h.mLongValue)

    def testCallWithBigLong(self):
        h = self.__jp.Test1()
        if sys.version > '3':
            l = int(4398046511103)
        else:
            l = long(4398046511103)
 
        self.assertRaises(TypeError, h.setByte, l)
        self.assertRaises(TypeError, h.setShort, l)
        self.assertRaises(TypeError, h.setInt, l)
        h.setLong(l)
        self.assertEqual(l, h.mLongValue)

    def testCallWithBigInt(self):
        h = self.__jp.Test1()
        if sys.version > '3' or sys.maxint > 2**31:
            l = int(4398046511103)
        else:
            l = long(4398046511103)
 
        self.assertRaises(TypeError, h.setByte, l)
        self.assertRaises(TypeError, h.setShort, l)
        self.assertRaises(TypeError, h.setInt, l)
        h.setLong(l)
        self.assertEqual(l, h.mLongValue)

    def testSetBoolean(self):
        h = self.__jp.Test1()
        self.assertEqual(False, h.mBooleanValue)
        h.setBoolean(True)
        self.assertEqual(True, h.mBooleanValue)
        h.setBoolean(False)
        self.assertEqual(False, h.mBooleanValue)
        # just testing the status quo, not sure about if this is nice
        h.setBoolean(42)
        self.assertEqual(True, h.mBooleanValue)
        h.setBoolean(0)
        self.assertEqual(False, h.mBooleanValue)
        if sys.version > '3':
            l = int(4398046511103)
        else:
            l = long(4398046511103)
        h.setBoolean(l)
        self.assertEqual(True, h.mBooleanValue)
        if sys.version > '3':
            l = int(0)
        else:
            l = long(0)
        h.setBoolean(l)
        self.assertEqual(False, h.mBooleanValue)

    def testCreateDate(self):
        d = jpype.java.util.Date(1448799485000)
        self.assertEqual(1448799485000, d.getTime())
        
    def testCharAttribute(self):
        h = self.__jp.Test1()
        h.charValue = u'b'

        self.assertEqual(h.charValue, 'b')
        if sys.version < '3':
            exp_repr = "u'b'"
        else:
            exp_repr = "'b'"
        self.assertEqual(repr(h.charValue), exp_repr)

    def testGetPrimitiveType(self):
        Integer = jpype.JClass("java.lang.Integer")
        intType = Integer.TYPE

    def testDifferentiateClassAndObject(self):
        h = self.__jp.Test1()

        self.assertEqual(h.callWithSomething(self.__jp.Test1), u"Class")
        result = h.callWithSomething(jpype.JObject(self.__jp.Test1,
                                                   jpype.java.lang.Object))
        self.assertEqual(result, u"Object")

    def testToString(self):
        h = self.__jp.Test1()
        self.assertEqual(str(h), 'aaa')

    def testSuperToString(self):
        h = self.__jp.Test2()
        self.assertEqual(str(h), 'aaa')

#       def testStringToConversion(self):
#               try:
#                       jpype.ConversionConfig.string = False
#                       for i in range(1):
#                               h = self.__jp.Test1()
#
#                               start = time.time();
#                               for j in range(10):
#                                       ts = h.getBigString()
#                               stop = time.time()
#                               print ts.__class__, (stop-start), 'ms'
#
#                               # for comparison ... convert a string to JStrng and back
#                               s = "".join([" "]*1024*1024*5)
#                               start = time.time()
#                               js = JString(s)
#                               stop = time.time()
#                               print "converted to JString in", (stop-start), 'ms', len(s)
#
#                               start = time.time()
#                               cs = str(JString(s))
#                               print cs
#                               print "converted to String in", (stop-start), 'ms', len(cs), cs
#               finally:
#                       jpype.ConversionConfig.string = True

    def testComplexMethodOvlerloading(self):
        c = self.__jp.TestOverloadC()
        self.assertEqual(c.foo(1), "foo(int) in C: 1")
        self.assertEqual(c.foo(), "foo() in A")

    def testPassedObjectGetsCleanedUp(self):
        h = self.__jp.Test1()
        block_size = 1024 * 1024 * 10
        def allocate_then_free():
            byte_buffer = jpype.JClass('java.nio.ByteBuffer')
            inst = byte_buffer.allocate(1024 * 1024 * 100)
             # passing the object back to java seems to stop it being collected
            result = h.callWithSomething(inst)
        rt = jpype.java.lang.Runtime.getRuntime()
        free = rt.freeMemory()
        for x in range(0, 10 * free // block_size):
            allocate_then_free()
        

    def testSyntheticMethod(self):
        h = jpype.JClass('jpype.attr.SyntheticMethods$GenericImpl')()
        h.foo(jpype.java.util.ArrayList())
        


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************

import jpype
import logging
from os import path
import sys
try:
    import unittest2 as unittest
except ImportError:
    import unittest

CLASSPATH = None

class JPypeTestCase(unittest.TestCase) :
    def setUp(self):
        if not jpype.isJVMStarted():
            root = path.dirname(path.abspath(path.dirname(__file__)))
            jvm_path = jpype.getDefaultJVMPath()
            logger = logging.getLogger(__name__)
            logger.info("Running testsuite using JVM %s" % jvm_path)
            classpath_arg = "-Djava.class.path=%s"
            classpath_arg %= path.join(root, 'classes')
            jpype.startJVM(jvm_path, "-ea",
                           # "-Xcheck:jni",
                           "-Xmx256M", "-Xms16M", classpath_arg)
        self.jpype = jpype.JPackage('jpype')
        if sys.version < '3':
            self.assertCountEqual = self.assertItemsEqual

    def tearDown(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import codecs
import platform
from glob import glob
import warnings

from setuptools import setup
from setuptools import Extension
from setuptools.command.build_ext import build_ext


"""
this parameter is used to opt out numpy support in _jpype library
"""
if "--disable-numpy" in sys.argv:
    disabled_numpy = True
    sys.argv.remove("--disable-numpy")
else:
    disabled_numpy = False

class FeatureNotice(Warning):
    """ indicate notices about features """
    pass


def read_utf8(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    return codecs.open(filename, encoding='utf-8').read()

def find_sources():
    cpp_files = []
    for dirpath, dirnames, filenames in os.walk('native'):
        for filename in filenames:
            if filename.endswith('.cpp') or filename.endswith('.c'):
                cpp_files.append(os.path.join(dirpath, filename))
    return cpp_files


platform_specific = {
    'include_dirs': [
        os.path.join('native', 'common', 'include'),
        os.path.join('native', 'python', 'include'),
    ],
    'sources': find_sources(),
}

fallback_jni = os.path.join('native', 'jni_include')
# try to include JNI first from eventually given JAVA_HOME, then from distributed
java_home = os.getenv('JAVA_HOME', '')
found_jni = False
if os.path.exists(java_home):
    platform_specific['include_dirs'] += [os.path.join(java_home, 'include')]

    # check if jni.h can be found
    for d in platform_specific['include_dirs']:
        if os.path.exists(os.path.join(d, 'jni.h')):
            found_jni = True
            break

    if not found_jni:
        import warnings
        warnings.warn('Falling back to provided JNI headers, since your provided'
                      ' JAVA_HOME "%s" does not provide jni.h' % java_home)
        platform_specific['include_dirs'] += [fallback_jni]

else:
    platform_specific['include_dirs'] += [fallback_jni]

if sys.platform == 'win32':
    platform_specific['libraries'] = ['Advapi32']
    platform_specific['define_macros'] = [('WIN32', 1)]
    jni_md_platform = 'win32'

elif sys.platform == 'darwin':
    platform_specific['libraries'] = ['dl']
    platform_specific['define_macros'] = [('MACOSX', 1)]
    jni_md_platform = 'darwin'

elif sys.platform.startswith('linux'):
    platform_specific['libraries'] = ['dl']
    jni_md_platform = 'linux'

elif sys.platform.startswith('freebsd'):
    jni_md_platform = 'freebsd'

else:
    warnings.warn("Your platform is not being handled explicitly."
                  " It may work or not!", UserWarning)

if found_jni:
    platform_specific['include_dirs'] += \
        [os.path.join(java_home, 'include', jni_md_platform)]

# include this stolen from FindJNI.cmake
"""
FIND_PATH(JAVA_INCLUDE_PATH2 jni_md.h
${JAVA_INCLUDE_PATH}
${JAVA_INCLUDE_PATH}/win32
${JAVA_INCLUDE_PATH}/linux
${JAVA_INCLUDE_PATH}/freebsd
${JAVA_INCLUDE_PATH}/solaris
${JAVA_INCLUDE_PATH}/hp-ux
${JAVA_INCLUDE_PATH}/alpha
)"""
 


jpypeLib = Extension(name='_jpype', **platform_specific)

class my_build_ext(build_ext):
    """
    Override some behavior in extension building:

    1. Numpy:
        If not opted out, try to use NumPy and define macro 'HAVE_NUMPY', so arrays
        returned from Java can be wrapped efficiently in a ndarray.
    2. handle compiler flags for different compilers via a dictionary.
    3. try to disable warning ‘-Wstrict-prototypes’ is valid for C/ObjC but not for C++
    """

    # extra compile args
    copt = {'msvc': ['/EHsc'],
            'unix' : ['-ggdb'],
            'mingw32' : [],
           }
    # extra link args
    lopt = {
            'msvc': [],
            'unix': [],
            'mingw32' : [],
           }

    def initialize_options(self, *args):
        """omit -Wstrict-prototypes from CFLAGS since its only valid for C code."""
        import distutils.sysconfig
        cfg_vars = distutils.sysconfig.get_config_vars()
        if 'CFLAGS' in cfg_vars:
            cfg_vars['CFLAGS'] = cfg_vars['CFLAGS'].replace('-Wstrict-prototypes', '')

        build_ext.initialize_options(self)

    def _set_cflags(self):
        # set compiler flags
        c = self.compiler.compiler_type
        if c in self.copt:
            for e in self.extensions:
                e.extra_compile_args = self.copt[ c ]
        if c in self.lopt:
            for e in self.extensions:
                e.extra_link_args = self.lopt[ c ]

    def build_extensions(self):
        self._set_cflags()
        # handle numpy
        if not disabled_numpy:
            try:
                import numpy
                jpypeLib.define_macros.append(('HAVE_NUMPY', 1))
                jpypeLib.include_dirs.append(numpy.get_include())
                warnings.warn("Turned ON Numpy support for fast Java array access",
                               FeatureNotice)
            except ImportError:
                pass
        else:
            warnings.warn("Turned OFF Numpy support for fast Java array access",
                          FeatureNotice)

        # has to be last call
        build_ext.build_extensions(self)

setup(
    name='JPype1',
    version='0.6.2',
    description='A Python to Java bridge.',
    long_description=(read_utf8('README.rst') + '\n\n' +
                      read_utf8('doc/CHANGELOG.rst') + '\n\n' +
                      read_utf8('AUTHORS.rst')),
    license='License :: OSI Approved :: Apache Software License',
    author='Steve Menard',
    author_email='devilwolf@users.sourceforge.net',
    maintainer='Luis Nell',
    maintainer_email='cooperate@originell.org',
    url='https://github.com/originell/jpype/',
    platforms=[
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows :: Windows 7',
        'Operating System :: Microsoft :: Windows :: Windows Vista',
        'Operating System :: POSIX :: Linux',
    ],
    classifiers=[
        'Programming Language :: Java',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ],
    packages=[
        'jpype', 'jpype.awt', 'jpype.awt.event', 'jpypex', 'jpypex.swing'],
    package_dir={
        'jpype': 'jpype',
        'jpypex': 'jpypex',
    },
    extras_require = {'numpy' : ['numpy>=1.6']},
    cmdclass={'build_ext': my_build_ext},
    zip_safe=False,
    ext_modules=[jpypeLib],
)



import jpype
from . import common

class CollectionTestCase(common.JPypeTestCase):

    def setUp(self):
        super(CollectionTestCase, self).setUp()

    def testCollection(self):
        collection = jpype.java.util.ArrayList()
        collection.add(1)
        collection.add(2)
        self.assertEqual([1, 2], [i for i in collection])

    def testIterateHashmap(self):
        collection = jpype.java.util.HashMap()
        collection.put('A',1)
        collection.put('B',2)
        asdict = dict()
        for x in collection.entrySet():
            asdict[x.getKey()] = x.getValue().longValue()
        self.assertEqual({'A':1,'B':2}, asdict)
                    
    def testEnumMap(self):
        enumclass = jpype.JClass('jpype.collection.TestEnum')
        enummap = jpype.java.util.EnumMap(enumclass)
        enummap.put(enumclass.A, 'ABC')
        enummap.put(enumclass.B, 'DEF')
        asdict = dict()
        for x in enummap.entrySet():
            asdict[str(x.getKey())] = x.getValue()
        self.assertEqual({'A':'ABC','B':'DEF'}, asdict)


from os import path
import shutil
import tempfile
from jpype import *

lucene_jar = path.join("..", "build", "lucene-1.4.3.jar")
if not path.isfile(lucene_jar):
    raise IOError, "Please provide %s" % path.abspath(lucene_jar)
startJVM(getDefaultJVMPath(),'-Djava.class.path=%s' % lucene_jar)

QueryParser = JClass("org.apache.lucene.queryParser.QueryParser")
IndexSearcher = JClass("org.apache.lucene.search.IndexSearcher")
IndexReader = JClass("org.apache.lucene.index.IndexReader")
StandardAnalyzer = JClass("org.apache.lucene.analysis.standard.StandardAnalyzer")
FSDirectory = JClass("org.apache.lucene.store.FSDirectory")
IndexWriter = JClass("org.apache.lucene.index.IndexWriter")
SimpleAnalyzer = JClass("org.apache.lucene.analysis.SimpleAnalyzer")

tmppath = tempfile.mkdtemp()
IndexWriter(tmppath, SimpleAnalyzer(), True).close()

directory = FSDirectory.getDirectory(tmppath, False)
reader = IndexReader.open(directory)
searcher = IndexSearcher(reader)
queryparser = QueryParser.parse("wenger","contents",StandardAnalyzer())
print queryparser.rewrite
print queryparser.rewrite.matchReport(reader)
qp = queryparser.rewrite(reader)
print qp
print searcher.search.matchReport(qp)
hits = searcher.search(qp)

shutil.rmtree(tmppath)



#*****************************************************************************
#   Copyright 2013 Thomas Calmant
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************

import os

# ------------------------------------------------------------------------------

class JVMNotFoundException(RuntimeError):
    pass

class JVMNotSupportedException(RuntimeError):
    pass

class JVMFinder(object):
    """
    JVM library finder base class
    """
    def __init__(self):
        """
        Sets up members
        """
        # Library file name
        self._libfile = "libjvm.so"

        # Predefined locations
        self._locations = ("/usr/lib/jvm", "/usr/java")

        # Search methods
        self._methods = (self._get_from_java_home,
                         self._get_from_known_locations)


    def find_libjvm(self, java_home):
        """
        Recursively looks for the given file

        :param java_home: A Java home folder
        :param filename: Name of the file to find
        :return: The first found file path, or None
        """
        found_jamvm = False
        non_supported_jvm = ('cacao', 'jamvm')
        found_non_supported_jvm = False
        # Look for the file
        for root, _, names in os.walk(java_home):
            if self._libfile in names:
                # Found it, but check for non supported jvms
                candidate = os.path.split(root)[1]
                if candidate in non_supported_jvm:
                    found_non_supported_jvm = True
                    continue # maybe we will find another one?
                return os.path.join(root, self._libfile)

        else:
            if found_non_supported_jvm:
                raise JVMNotSupportedException("Sorry '{0}' is known to be "
                                               "broken. Please ensure your "
                                               "JAVA_HOME contains at least "
                                               "another JVM implementation "
                                               "(eg. server)"
                                               .format(candidate))
            # File not found
            raise JVMNotFoundException("Sorry no JVM could be found. "
                                       "Please ensure your JAVA_HOME "
                                       "environment variable is pointing "
                                       "to correct installation.")


    def find_possible_homes(self, parents):
        """
        Generator that looks for the first-level children folders that could be
        Java installations, according to their name

        :param parents: A list of parent directories
        :return: The possible JVM installation folders
        """
        homes = []
        java_names = ('jre', 'jdk', 'java')

        for parent in parents:
            for childname in sorted(os.listdir(parent)):
                # Compute the real path
                path = os.path.realpath(os.path.join(parent, childname))
                if path in homes or not os.path.isdir(path):
                    # Already known path, or not a directory -> ignore
                    continue

                # Check if the path seems OK
                real_name = os.path.basename(path).lower()
                for java_name in java_names:
                    if java_name in real_name:
                        # Correct JVM folder name
                        homes.append(path)
                        yield path
                        break


    def get_jvm_path(self):
        """
        Retrieves the path to the default or first found JVM library

        :return: The path to the JVM shared library file
        :raise ValueError: No JVM library found
        """
        for method in self._methods:
            try:
                jvm = method()
            except NotImplementedError:
                # Ignore missing implementations
                pass
            except JVMNotFoundException:
                # Ignore not successful methods
                pass

            else:
                if jvm is not None:
                    return jvm

        else:
            raise JVMNotFoundException("No JVM shared library file ({0}) "
                                       "found. Try setting up the JAVA_HOME "
                                       "environment variable properly."
                                       .format(self._libfile))


    def _get_from_java_home(self):
        """
        Retrieves the Java library path according to the JAVA_HOME environment
        variable

        :return: The path to the JVM library, or None
        """
        # Get the environment variable
        java_home = os.getenv("JAVA_HOME")
        if java_home and os.path.exists(java_home):
            # Get the real installation path
            java_home = os.path.realpath(java_home)

            # Look for the library file
            return self.find_libjvm(java_home)


    def _get_from_known_locations(self):
        """
        Retrieves the first existing Java library path in the predefined known
        locations

        :return: The path to the JVM library, or None
        """
        for home in self.find_possible_homes(self._locations):
            jvm = self.find_libjvm(home)
            if jvm is not None:
                return jvm



from jpype import *
import time

def pySubscriber (proxy,
                  javaNamingFactory="weblogic.jndi.WLInitialContextFactory",
                  javaNamingProvider="t3://158.188.40.21:7001",
                  connectionFactory="weblogic.jms.ConnectionFactory",
                  topicName="defaultTopic"):
    return messaging.JpypeSubscriber(proxy,javaNamingFactory,javaNamingProvider,
                                    connectionFactory,topicName)

## Startup Jpype and import the messaging java package
startJVM("C:\\program files\\Java\\j2re1.4.2_02\\bin\\client\\jvm.dll",
    "-Djava.class.path=D:/jIRAD/JpypeJMS/src;D:/jIRAD/JpypeJMS/classes;C:/bea/weblogic81/server/lib/weblogic.jar")
messaging = JPackage('messaging')

# Setup the JProxy for the messaging.JpypeSubscriberCallback interface
class pyCallback:
    startTime = 0
    count = 0
    def onMessage (self,text):
        print text
        if text=='Start':
            pyCallback.startTime=time.time()
            pyCallback.count = 0
        elif text=='Stop':
            print "Message Rate =",float(pyCallback.count)/(time.time()-pyCallback.startTime)
        else:
            pyCallback.count += 1

c = pyCallback()
proxy = JProxy(messaging.JpypeSubscriberCallback,inst=c)

# Get a subscriber
sub = pySubscriber(proxy)
print "Listening..."

# Prevent this thread from exiting
time.sleep(1000)

# exit
shutdownJVM()



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import sys
import jpype
from jpype import JPackage, JArray, JByte, java
from . import common

if sys.version > '3':
    unicode = str


def haveNumpy():
    try:
        import numpy
        return True
    except ImportError:
        return False


class ArrayTestCase(common.JPypeTestCase):

    def setUp(self):
        common.JPypeTestCase.setUp(self)
        self.VALUES = [12234, 1234, 234, 1324, 424, 234, 234, 142, 5, 251, 242, 35, 235, 62,
                       1235, 46, 245132, 51, 2, 3, 4]

    def testReadArray(self):
        t = JPackage("jpype").array.TestArray()
        self.assertNotIsInstance(t, JPackage)

        self.assertCountEqual(self.VALUES, t.i)

        self.assertEqual(t.i[0], self.VALUES[0])
        self.assertCountEqual(self.VALUES[1:-2], t.i[1:-2])

    def testStangeBehavior(self):
        ''' Test for stange crash reported in bug #1089302'''
        Test2 = jpype.JPackage('jpype.array').Test2
        test = Test2()
        test.test(test.getValue())

    def testWriteArray(self):
        t = JPackage("jpype").array.TestArray()
        self.assertNotIsInstance(t, JPackage)

        t.i[0] = 32
        self.assertEqual(t.i[0], 32)

        t.i[1:3] = (33, 34)
        self.assertEqual(t.i[1], 33)
        self.assertEqual(t.i[2], 34)

        self.assertCountEqual(t.i[:5], (32, 33, 34, 1324, 424))

    def testObjectArraySimple(self):
        a = JArray(java.lang.String, 1)(2)
        a[1] = "Foo"
        self.assertEqual("Foo", a[1])

    def testByteArraySimple(self):
        a = JArray(JByte)(2)
        a[1] = 2
        self.assertEqual(a[1], 2)

    def testIterateArray(self):
        t = JPackage("jpype").array.TestArray()
        self.assertFalse(isinstance(t, JPackage))

        for i in t.i:
            self.assertNotEqual(i, 0)

    def testGetSubclass(self):
        t = JPackage("jpype").array.TestArray()
        v = t.getSubClassArray()

        self.assertTrue(isinstance(v[0], unicode))

    def testGetArrayAsObject(self):
        t = JPackage("jpype").array.TestArray()
        v = t.getArrayAsObject()

    def testCharArrayAsString(self):
        t = JPackage("jpype").array.TestArray()
        v = t.charArray
        self.assertEqual(str(v), 'avcd')
        self.assertEqual(unicode(v), u'avcd')

    def testByteArrayAsString(self):
        t = JPackage("jpype").array.TestArray()
        v = t.byteArray
        self.assertEqual(str(v), 'avcd')

    def testByteArrayIntoVector(self):
        ba = jpype.JArray(jpype.JByte)(b'123')
        v = jpype.java.util.Vector(1)
        v.add(ba)
        self.assertEqual(len(v), 1)
        self.assertNotEqual(v[0], None)

    def testJArrayConversionBool(self):
        expected = [True, False, False, True]
        jarr = jpype.JArray(jpype.JBoolean)(expected)

        self.assertCountEqual(expected, jarr[:])

    def testJArrayConversionChar(self):
        t = JPackage("jpype").array.TestArray()
        v = t.charArray
        self.assertEqual(v[:], 'avcd')
        # FIXME: this returns unicode on windows
        self.assertEqual(str(v[:]), 'avcd')
        self.assertEqual(unicode(v[:]), u'avcd')

    def testJArrayConversionByte(self):
        expected = (0, 1, 2, 3)
        ByteBuffer = jpype.java.nio.ByteBuffer
        bb = ByteBuffer.allocate(4)
        buf = bb.array()
        for i in range(len(expected)):
            buf[i] = expected[i]

        self.assertCountEqual(expected[:], buf[:])

    def testJArrayConversionShort(self):
        # filter out values, which can not be converted to jshort
        self.VALUES = [v for v in self.VALUES if v < (2**16 / 2 - 1)
                       and v > (2**16 / 2 * -1)]
        jarr = jpype.JArray(jpype.JShort)(self.VALUES)
        result = jarr[0: len(jarr)]
        self.assertCountEqual(self.VALUES, result)

        result = jarr[2:10]
        self.assertCountEqual(self.VALUES[2:10], result)

        # TODO: investigate why overflow is being casted on linux, but not on windows
        # with self.assertRaises(jpype._):
        #    jpype.JArray(jpype.JShort)([2**16/2])

    def testJArrayConversionInt(self):
        jarr = jpype.JArray(jpype.JInt)(self.VALUES)
        result = jarr[0: len(jarr)]
        self.assertCountEqual(self.VALUES, result)

        result = jarr[2:10]
        self.assertCountEqual(self.VALUES[2:10], result)

    def testJArrayConversionLong(self):
        jarr = jpype.JArray(jpype.JLong)(self.VALUES)
        result = jarr[0: len(jarr)]
        self.assertCountEqual(self.VALUES, result)

        result = jarr[2:10]
        self.assertCountEqual(self.VALUES[2:10], result)

    def testJArrayConversionFloat(self):
        VALUES = [float(x) for x in self.VALUES]
        jarr = jpype.JArray(jpype.JFloat)(VALUES)
        result = jarr[0: len(jarr)]
        self.assertCountEqual(jarr, result)

        result = jarr[2:10]
        self.assertCountEqual(VALUES[2:10], result)

    def testJArrayConversionDouble(self):
        VALUES = [float(x) for x in self.VALUES]
        jarr = jpype.JArray(jpype.JDouble)(VALUES)
        self.assertCountEqual(VALUES, jarr)
        result = jarr[:]
        self.assertCountEqual(VALUES, result)

        result = jarr[2:10]

        self.assertEqual(len(VALUES[2:10]), len(result))
        self.assertCountEqual(VALUES[2:10], result)

        # empty slice
        result = jarr[-1:3]
        expected = VALUES[-1:3]
        self.assertCountEqual(expected, result)

        result = jarr[3:-2]
        expected = VALUES[3:-2]
        self.assertCountEqual(expected, result)

    def testConversionError(self):
        jarr = jpype.JArray(jpype.JInt, 1)(10)
        with self.assertRaises(RuntimeError):
            jarr[1:2] = [dict()]

        # -1 is returned by python, if conversion fails also, ensure this works
        jarr[1:2] = [-1]

    def testObjectArrayInitial(self):
        l1 = java.util.ArrayList()
        l1.add(0)
        l2 = java.util.ArrayList()
        l2.add(42)
        l3 = java.util.ArrayList()
        l3.add(13)
        jarr = jpype.JArray(java.util.ArrayList, 1)([l1, l2, l3])

        self.assertEqual(l1, jarr[0])
        self.assertEqual(l2, jarr[1])
        self.assertEqual(l3, jarr[2])

    @unittest.skipUnless(haveNumpy(), "numpy not available")
    def testSetFromNPBoolArray(self):
        import numpy as np
        n = 100
        a = np.random.randint(0, 1, size=n).astype(np.bool)
        jarr = jpype.JArray(jpype.JBoolean)(n)
        jarr[:] = a
        self.assertCountEqual(a, jarr)

    @unittest.skipUnless(haveNumpy(), "numpy not available")
    def testSetFromNPByteArray(self):
        import numpy as np
        n = 100
        a = np.random.randint(-128, 127, size=n).astype(np.byte)
        jarr = jpype.JArray(jpype.JByte)(n)
        jarr[:] = a
        self.assertCountEqual(a, jarr)

    @unittest.skipUnless(haveNumpy(), "numpy not available")
    def testSetFromNPShortArray(self):
        import numpy as np
        n = 100
        a = np.random.randint(-32768, 32767, size=n).astype(np.short)
        jarr = jpype.JArray(jpype.JShort)(n)
        jarr[:] = a
        self.assertCountEqual(a, jarr)

    @unittest.skipUnless(haveNumpy(), "numpy not available")
    def testSetFromNPIntArray(self):
        import numpy as np
        n = 100
        a = np.random.randint(-2**31 - 1, 2**31 - 1, size=n).astype(np.int32)
        jarr = jpype.JArray(jpype.JInt)(n)
        jarr[:] = a
        self.assertCountEqual(a, jarr)

    @unittest.skipUnless(haveNumpy(), "numpy not available")
    def testSetFromNPLongArray(self):
        import numpy as np
        n = 100
        # actuall the lower bound should be -2**63 -1, but raises Overflow
        # error in numpy
        a = np.random.randint(-2**63, 2**63 - 1, size=n).astype(np.int64)
        jarr = jpype.JArray(jpype.JLong)(n)
        jarr[:] = a
        self.assertCountEqual(a, jarr)

    @unittest.skipUnless(haveNumpy(), "numpy not available")
    def testSetFromNPFloatArray(self):
        import numpy as np
        n = 100
        a = np.random.random(n).astype(np.float32)
        jarr = jpype.JArray(jpype.JFloat)(n)
        jarr[:] = a
        self.assertCountEqual(a, jarr)

    @unittest.skipUnless(haveNumpy(), "numpy not available")
    def testSetFromNPDoubleArray(self):
        import numpy as np
        n = 100
        a = np.random.random(n).astype(np.float64)
        jarr = jpype.JArray(jpype.JDouble)(n)
        jarr[:] = a
        self.assertCountEqual(a, jarr)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from ._jpackage import *
from ._jclass import *
from ._jarray import *
from ._jwrapper import *
from ._jproxy import *
from ._jexception import *
from ._core import *
from ._gui import *

from . import JClassUtil

__version_info__ = (0, 6, 2)
__version__ = ".".join(str(i) for i in __version_info__)

def JIterator(it):
    """Deprecated"""
    return it

java = JPackage("java")
javax = JPackage("javax")



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#          http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
from . import _jclass

def _initialize():
    _jclass.registerClassCustomizer(ComparableCustomizer())

class ComparableCustomizer(object):
    _METHODS = {
            "__cmp__": lambda self, o: self.compareTo(o)
    }

    def canCustomize(self, name, jc):
        return name == 'java.lang.Comparable'

    def customize(self, name, jc, bases, members):
        members.update(ComparableCustomizer._METHODS)
